/*****************************************************************************
* Sistemas Integrados TECNISA                                                *
*                                                                            *
* Compras e Suprimentos                                                      *
*                                                                            *
******************************************************************************
*                                                                            *
* Nome: SEMANAL_DATA_ENTR_MES                                                *
*                                                                            *
* Descrio:                                                                 *
* Calcula datas de entrega para um determinado mes baseando-se na            *
* Periodicidade de Entrega e nos Feriados de Compras.                        *
*                                                                            *
*                                                                            *
* Criao:                                                                   *
* 08/01/99 - Yoris L. de Souza                                               *
*                                                                            *
* Manutenes:                                                               *
* 18/11/99 - Yoris L. de Souza                                               *
*            Inclusao de verificacao de existencia de uma data ja calculada  *
*            devido a um dia nao util.                                       *
*                                                                            *
******************************************************************************

******************************************************************************
* PARAMETROS:                                                                *
*                                                                            *
* P_CDFILIAL     - Cdigo da Filial                                          *
* P_CDPERIENTR   - Cdigo da Periodicidade de Entrega                        *
* P_DTINICIAL    - Data do Inicio do Mes                                     *
* P_IDDIANUTIL   - Considera dia nao util                                    *
******************************************************************************
* @D:\SQL\CALECOMP\8040_CMPSTP_SEM_DATA_ENTR_MES.SQL                         *
******************************************************************************/

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE 
            o.name = 'SEMANAL_DATA_ENTR_MES'  and 
            type = 'P'                  and
            u.uid = o.uid               and 
            u.name = USER)
  DROP PROCEDURE SEMANAL_DATA_ENTR_MES
  GO

CREATE PROCEDURE SEMANAL_DATA_ENTR_MES
   @P_CDFILIAL     VARCHAR(4),
   @P_CDPERIENTR   VARCHAR(2),
   @P_DTINICIAL    DATETIME,
   @P_IDDIANUTIL   VARCHAR(1),
   @P_DTPROCESSA   DATETIME
AS
   DECLARE @V_CONTAUXI   NUMERIC(10);
   DECLARE @V_DIENTREGA  INT;
   DECLARE @V_DTDIAENTR  DATETIME;
   DECLARE @V_IDDIASEM   INT;
   DECLARE @V_DTENTRAUX  DATETIME;
   DECLARE @V_PRIMVEZ    NUMERIC(10);
   DECLARE @V_JAEXISTE   NUMERIC(10);
   DECLARE @V_DTAUX      DATETIME;
   DECLARE @V_DTVERAUX   DATETIME;
   DECLARE @V_DTENTREGA_PROX DATETIME;
   DECLARE @V_DTENTREGA_ANT DATETIME;
   DECLARE @V_EXISTENTR INT;

/*-------- CURSOR QUE CONTEM AS DATAS DE ENTREGA NO MES COM BASE NOS DIAS DE ENTREGA SEMANAIS ------*/

 DECLARE C_DATAS_ENTR CURSOR FOR
   SELECT
      CONVERT(INT,DIENTREGA),
      1 AS OCORR
    FROM DIASENTR
    WHERE
      CDPERIENTR = @P_CDPERIENTR AND
      IDOCOR01 = '1'
   UNION
   SELECT
     CONVERT(INT,DIENTREGA),
     2  AS OCORR
   FROM DIASENTR
     WHERE
      CDPERIENTR = @P_CDPERIENTR AND
      IDOCOR02 = '1'
   UNION
   SELECT
     CONVERT(INT,DIENTREGA),
     3  AS OCORR
   FROM DIASENTR
   WHERE
     CDPERIENTR = @P_CDPERIENTR AND
     IDOCOR03 = '1'
   UNION
   SELECT
     CONVERT(INT,DIENTREGA),
     4 AS OCORR
   FROM DIASENTR
   WHERE
     CDPERIENTR = @P_CDPERIENTR AND
     IDOCOR04 = '1'
   UNION
   SELECT
     CONVERT(INT,DIENTREGA),
     5 AS OCORR
   FROM DIASENTR
   WHERE
     CDPERIENTR = @P_CDPERIENTR AND
     IDOCOR05 = '1'
   ORDER BY 2;

BEGIN
    SET @V_DTENTREGA_ANT = NULL;
    SET @V_DTENTRAUX = @P_DTINICIAL;
    SET @V_DTDIAENTR = @P_DTINICIAL;
    OPEN C_DATAS_ENTR;
    FETCH NEXT FROM C_DATAS_ENTR INTO @V_DIENTREGA, @V_IDDIASEM;
    WHILE (@@FETCH_STATUS = 0)
    BEGIN
       IF @V_IDDIASEM <> 5
       BEGIN
          SET @V_DTDIAENTR = @P_DTINICIAL - 1;
       END;
       SET @V_DTDIAENTR = DATEADD(MM, 1, @V_DTDIAENTR) - DAY(DATEADD(MM, 1, @V_DTDIAENTR));/*LAST_DAY*/
       IF @V_IDDIASEM = 5
       BEGIN
          SET @V_DTDIAENTR = @V_DTDIAENTR - 7;
          EXEC PROXIMO_DIA @V_DIENTREGA,1,@P_DTPROXIMA = @V_DTDIAENTR OUTPUT;
       END
       ELSE
       BEGIN
          EXEC PROXIMO_DIA @V_DIENTREGA,@V_IDDIASEM,@P_DTPROXIMA = @V_DTDIAENTR OUTPUT;
       END;

       IF ((((MONTH(@V_DTDIAENTR) = MONTH(@P_DTINICIAL)) AND (@V_IDDIASEM > 1)) OR
           (@V_IDDIASEM = 1)) AND (@V_DTDIAENTR >= @P_DTPROCESSA))
       BEGIN

  /*- CALCULA UMA DATA DE ENTREGA ANTERIOR A UMA DATA NAO UTIL -*/
          IF (@P_IDDIANUTIL = 'S')
          BEGIN
            SET @V_CONTAUXI = 1;
            WHILE (@V_CONTAUXI > 0)
            BEGIN
              SET @V_CONTAUXI = 0;
              SELECT @V_CONTAUXI = COUNT(*)
              FROM FERICOMP
              WHERE
              CDFILIAL   = @P_CDFILIAL AND
              DTFERICOMP = @V_DTDIAENTR;
              IF (@V_CONTAUXI > 0)
              BEGIN
                 SET @V_DTDIAENTR = @V_DTDIAENTR - 1;
              END;
            END;
          END;
 /* - Verifica se j existe a data de entrega que foi calculada por um dia no util -*/
          SET @V_JAEXISTE = 0;
          SELECT @V_JAEXISTE = COUNT(*)
          FROM CALECOMP
          WHERE
            CDFILIAL   = @P_CDFILIAL AND
            CDPERIENTR = @P_CDPERIENTR AND
            DTENTREGA  = @V_DTDIAENTR;
        /*- Se j existir a data pula -*/
          IF (@V_JAEXISTE = 0)
          BEGIN
        /*- VERIFICA SE JA EXISTE ALGUMA DATA DE ENTREGA CALCULADA PARA  -*/
        /*- INCLUIR A DATA ANTERIOR A PROXIMA ENTREGA  -*/
              SET @V_PRIMVEZ = 0;
              SELECT @V_PRIMVEZ = COUNT(*)
              FROM CALECOMP
              WHERE
              CDFILIAL   = @P_CDFILIAL AND
              CDPERIENTR = @P_CDPERIENTR AND
              DTENTREGA  = @V_DTENTRAUX;

        /*- INCLUI A DATA ANTERIOR A PROXIMA ENTREGA  -*/
              IF (@V_PRIMVEZ > 0)
              BEGIN
                 UPDATE CALECOMP
                  SET DTANTPROXENT = @V_DTDIAENTR - 1
                  WHERE CDFILIAL   = @P_CDFILIAL AND
                       CDPERIENTR = @P_CDPERIENTR AND
                       DTENTREGA  = @V_DTENTRAUX;
              END;

        /*- INCLUI A DATA DE ENTREGA CALCULADA NO CALENDARIO DE COMPRAS -*/
              INSERT INTO CALECOMP
                (CDFILIAL,
                 CDPERIENTR,
                 DTENTREGA,
                 DTANTPROXENT)
                VALUES
                (@P_CDFILIAL,
                 @P_CDPERIENTR,
                 @V_DTDIAENTR,
                 @V_DTENTRAUX);
              SET @V_DTENTRAUX = @V_DTDIAENTR;
/*
          UPDATE CALECOMP
          SET DTANTPROXENT = @V_DTDIAENTR - 1
          WHERE
            CDFILIAL   = @P_CDFILIAL AND
            CDPERIENTR = @P_CDPERIENTR AND
            DTENTREGA  = @V_DTENTREGA_ANT;
*/
          SET @V_DTENTREGA_ANT = @V_DTDIAENTR;

          END;
       END;
       FETCH NEXT FROM C_DATAS_ENTR INTO @V_DIENTREGA, @V_IDDIASEM;
    END;
    CLOSE C_DATAS_ENTR;
    DEALLOCATE C_DATAS_ENTR;

    DECLARE C_CALECOMP CURSOR FOR
       SELECT
          DTENTREGA
       FROM
          CALECOMP
       WHERE
          CDFILIAL   = @P_CDFILIAL AND
          CDPERIENTR = @P_CDPERIENTR
       ORDER BY
          DTENTREGA;

    SET @V_DTENTREGA_ANT = NULL;

    OPEN C_CALECOMP;
    FETCH NEXT FROM C_CALECOMP INTO @V_DTENTRAUX;
    WHILE (@@FETCH_STATUS = 0)
    BEGIN
       UPDATE CALECOMP
       SET DTANTPROXENT = @V_DTENTRAUX - 1
       WHERE CDFILIAL   = @P_CDFILIAL AND
             CDPERIENTR = @P_CDPERIENTR AND
             DTENTREGA  = @V_DTENTREGA_ANT;

       SET @V_DTENTREGA_ANT = @V_DTENTRAUX;
       FETCH NEXT FROM C_CALECOMP INTO @V_DTENTRAUX;
    END;
    CLOSE C_CALECOMP;
    DEALLOCATE C_CALECOMP;

    SET @V_DTENTRAUX = @P_DTINICIAL;

    SET @V_DTENTRAUX = DATEADD(MM, 1, @V_DTENTRAUX) - DAY(DATEADD(MM, 1, @V_DTENTRAUX));/*LAST_DAY*/

    SET @V_DTENTRAUX = @V_DTENTRAUX + 1;

  /*- CALCULA UMA DATA DA PROXIMA ENTREGA ANTERIOR A UMA DATA NAO UTIL -*/
    IF (@P_IDDIANUTIL = 'S')
    BEGIN
      SET @V_CONTAUXI = 1;
      WHILE (@V_CONTAUXI > 0)
      BEGIN
        SET @V_CONTAUXI = 0;
        SELECT @V_CONTAUXI = COUNT(*)
        FROM FERICOMP
        WHERE
        CDFILIAL   = @P_CDFILIAL AND
        DTFERICOMP = @V_DTENTRAUX;
        IF (@V_CONTAUXI > 0)
        BEGIN
           SET @V_DTENTRAUX = @V_DTENTRAUX + 1;
        END;
      END;
    END;

    SET @V_EXISTENTR = 0
    WHILE (@V_EXISTENTR <> 1) AND (@V_EXISTENTR <> 70)
    BEGIN
      SET @V_EXISTENTR = 0;
      SELECT
        @V_EXISTENTR = COUNT(*)
      FROM DIASENTR
      WHERE
        CDPERIENTR = @P_CDPERIENTR AND
        CONVERT(INT, DIENTREGA) = DATEPART(DW, @V_DTENTRAUX)

      IF @V_EXISTENTR = 0 BEGIN
         SET @V_EXISTENTR = @V_EXISTENTR + 10;
         SET @V_DTENTRAUX = @V_DTENTRAUX + 1;
      END;
    END;

    UPDATE CALECOMP
    SET DTANTPROXENT = @V_DTENTRAUX - 1
    WHERE
      CDFILIAL   = @P_CDFILIAL AND
      CDPERIENTR = @P_CDPERIENTR AND
      DTENTREGA  = @V_DTENTREGA_ANT;
END;
GO
